<?php

declare(strict_types=1);

namespace spec\Dedi\SyliusVoucherPlugin\Checker;

use Dedi\SyliusVoucherPlugin\Checker\OrderPaymentMethodSelectionRequirementChecker;
use Dedi\SyliusVoucherPlugin\Model\OrderInterface;
use Dedi\SyliusVoucherPlugin\Specification\IsOrderFullyPaidUsingVoucherSpecification;
use PhpSpec\ObjectBehavior;
use Sylius\Component\Core\Checker\OrderPaymentMethodSelectionRequirementCheckerInterface;

class OrderPaymentMethodSelectionRequirementCheckerSpec extends ObjectBehavior
{
    function let(
        OrderPaymentMethodSelectionRequirementCheckerInterface $baseChecker,
        IsOrderFullyPaidUsingVoucherSpecification $isOrderFullyPaidUsingVoucherSpecification
    ) {
        $this->beConstructedWith(
            $baseChecker,
            $isOrderFullyPaidUsingVoucherSpecification
        );
    }

    function it_is_initializable()
    {
        $this->shouldHaveType(OrderPaymentMethodSelectionRequirementChecker::class);
    }

    function it_requires_payment_when_base_cheker_requires_it(
        OrderPaymentMethodSelectionRequirementCheckerInterface $baseChecker,
        OrderInterface $order
    ) {
        $baseChecker->isPaymentMethodSelectionRequired($order)->willReturn(true);

        $this->isPaymentMethodSelectionRequired($order)->shouldReturn(true);
    }

    function it_requires_payment_when_base_cheker_does_not_but_order_is_paid_using_vouchers(
        OrderPaymentMethodSelectionRequirementCheckerInterface $baseChecker,
        IsOrderFullyPaidUsingVoucherSpecification $isOrderFullyPaidUsingVoucherSpecification,
        OrderInterface $order
    ) {
        $baseChecker->isPaymentMethodSelectionRequired($order)->willReturn(false);
        $isOrderFullyPaidUsingVoucherSpecification->__invoke($order)->willReturn(true);

        $this->isPaymentMethodSelectionRequired($order)->shouldReturn(true);
    }

    function it_does_not_require_payment_when_neither_base_cheker_requires_it_nor_the_order_is_paid_using_vouchers(
        OrderPaymentMethodSelectionRequirementCheckerInterface $baseChecker,
        IsOrderFullyPaidUsingVoucherSpecification $isOrderFullyPaidUsingVoucherSpecification,
        OrderInterface $order
    ) {
        $baseChecker->isPaymentMethodSelectionRequired($order)->willReturn(false);
        $isOrderFullyPaidUsingVoucherSpecification->__invoke($order)->willReturn(false);

        $this->isPaymentMethodSelectionRequired($order)->shouldReturn(false);
    }
}
