<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Api\Controller;

use ApiPlatform\Api\IriConverterInterface;
use Dedi\SyliusVoucherPlugin\Model\VoucherConfigurationImageInterface;
use Dedi\SyliusVoucherPlugin\Model\VoucherConfigurationInterface;
use Sylius\Component\Core\Model\ImageInterface;
use Sylius\Component\Core\Uploader\ImageUploaderInterface;
use Sylius\Component\Resource\Factory\FactoryInterface;
use Sylius\Component\Resource\Model\ResourceInterface;
use Sylius\Component\Resource\Repository\RepositoryInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Webmozart\Assert\Assert;

class UploadVoucherConfigurationImageAction extends AbstractController
{
    public function __construct(
        private readonly FactoryInterface $voucherConfigurationImageFactory,
        private readonly RepositoryInterface $voucherConfigurationImageRepository,
        private readonly ImageUploaderInterface $imageUploader,
        private readonly IriConverterInterface $iriConverter,
    ) {
    }

    public function __invoke(Request $request): ImageInterface
    {
        $file = $request->files->get('file');

        /** @var VoucherConfigurationImageInterface $image */
        $image = $this->voucherConfigurationImageFactory->createNew();
        $image->setFile($file);

        /** @var string $ownerIri */
        $ownerIri = $request->request->get('owner');
        Assert::notEmpty($ownerIri);

        /** @var ResourceInterface|VoucherConfigurationInterface $owner */
        $owner = $this->iriConverter->getResourceFromIri($ownerIri);
        Assert::isInstanceOf($owner, VoucherConfigurationInterface::class);

        $oldImage = $owner->getBackgroundImage();
        if ($oldImage !== null) {
            $this->voucherConfigurationImageRepository->remove($oldImage);
        }
        $owner->setBackgroundImage($image);

        $this->imageUploader->upload($image);

        return $image;
    }
}
