<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Checker;

use Dedi\SyliusVoucherPlugin\Exception\UnexpectedTypeException;
use Dedi\SyliusVoucherPlugin\Model\VoucherInterface;
use Sylius\Component\Order\Model\OrderInterface;

final class VoucherFilter implements VoucherFilterInterface
{
    public function filterEligibleVoucher(OrderInterface $order, iterable $rules, array $vouchers): array
    {
        /** @var array $voucherEligible */
        $voucherEligible = [];

        foreach ($rules as $key => $rule) {
            if (!$rule instanceof VoucherRuleInterface) {
                throw new UnexpectedTypeException($rule, VoucherRuleInterface::class);
            }

            foreach ($vouchers as $voucher) {
                if (!$voucher instanceof VoucherInterface) {
                    throw new UnexpectedTypeException($rule, VoucherInterface::class);
                }

                if ($rule->getRule($order, $voucher)) {
                    $voucherEligible[$voucher->getId()] = $voucher;
                }
            }
        }

        return $voucherEligible;
    }
}
