<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Controller\Action;

use Dedi\SyliusVoucherPlugin\Model\VoucherBalanceCollection;
use Dedi\SyliusVoucherPlugin\Repository\VoucherRepositoryInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * The purpose of this class is to show the voucher balance, i.e. what amount is still available on enabled vouchers
 */
final class VoucherBalanceAction extends AbstractController
{
    private readonly VoucherRepositoryInterface $voucherRepository;

    public function __construct(
        VoucherRepositoryInterface $voucherRepository,
    ) {
        $this->voucherRepository = $voucherRepository;
    }

    public function __invoke(Request $request): Response
    {
        $voucherBalanceCollection = VoucherBalanceCollection::createFromVouchers(
            $this->voucherRepository->findEnabledNotExpired(),
        );

        return $this->render('@DediSyliusVoucherPlugin/Admin/voucherBalance.html.twig', ['voucherBalanceCollection' => $voucherBalanceCollection]);
    }
}
