<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Form\Type;

use Sylius\Bundle\ResourceBundle\Form\Type\ResourceAutocompleteChoiceType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;

final class CustomerAutocompleteChoiceType extends AbstractType
{
    private readonly UrlGeneratorInterface $urlGenerator;

    public function __construct(UrlGeneratorInterface $urlGenerator)
    {
        $this->urlGenerator = $urlGenerator;
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'resource' => 'sylius.customer',
            'choice_name' => 'email',
            'choice_value' => 'email',
        ]);
    }

    public function buildView(FormView $view, FormInterface $form, array $options): void
    {
        $view->vars['remote_criteria_type'] = 'contains';
        $view->vars['remote_criteria_name'] = 'phrase';
        $view->vars['remote_url'] = $this->urlGenerator->generate('dedi_sylius_voucher_admin_ajax_customer_by_email_phrase');
        $view->vars['load_edit_url'] = $this->urlGenerator->generate('dedi_sylius_voucher_admin_ajax_customer_by_email');
    }

    public function getBlockPrefix(): string
    {
        return 'dedi_sylius_voucher_customer_autocomplete_choice';
    }

    public function getParent(): string
    {
        return ResourceAutocompleteChoiceType::class;
    }
}
