<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Modifier;

use Dedi\SyliusVoucherPlugin\Model\AdjustmentInterface;
use Dedi\SyliusVoucherPlugin\Model\OrderInterface;
use Dedi\SyliusVoucherPlugin\Model\VoucherInterface;
use Doctrine\ORM\EntityManagerInterface;
use RuntimeException;

/**
 * This class' responsibility is to modify the amount on a voucher when it's used to pay for an order
 */
final class OrderVoucherAmountModifier implements OrderVoucherAmountModifierInterface
{
    private readonly EntityManagerInterface $voucherManager;

    public function __construct(EntityManagerInterface $voucherManager)
    {
        $this->voucherManager = $voucherManager;
    }

    public function decrement(OrderInterface $order): void
    {
        foreach ($order->getAdjustments(AdjustmentInterface::ORDER_VOUCHER_ADJUSTMENT) as $adjustment) {
            $voucher = self::getVoucher($order, $adjustment->getOriginCode());

            $amount = abs($adjustment->getAmount());

            if ($amount >= $voucher->getAmount()) {
                $voucher->disable();
                $voucher->setAmount(0);
            }

            if ($amount < $voucher->getAmount()) {
                $voucher->enable();

                $voucher->setAmount($voucher->getAmount() - $amount);
            }
        }

        $this->voucherManager->flush();
    }

    public function increment(OrderInterface $order): void
    {
        foreach ($order->getAdjustments(AdjustmentInterface::ORDER_VOUCHER_ADJUSTMENT) as $adjustment) {
            $voucher = self::getVoucher($order, $adjustment->getOriginCode());

            $voucher->setAmount($voucher->getAmount() + abs($adjustment->getAmount()));

            if ($voucher->getAmount() > 0) {
                $voucher->enable();
            }
        }

        $this->voucherManager->flush();
    }

    private static function getVoucher(OrderInterface $order, string $code): VoucherInterface
    {
        foreach ($order->getVouchers() as $voucher) {
            if ($voucher->getCode() === $code) {
                return $voucher;
            }
        }

        throw new RuntimeException(sprintf('The order %s does not have a voucher with code %s', $order->getNumber(), $code));
    }
}
